import sys
import io
import click
import yaml
from dulwich.repo import Repo
from pathlib import Path
from importlib.machinery import SourceFileLoader
from cookiecutter.main import cookiecutter
from matador.cli.decorators import windows_only, deploys_changes
import matador.cli.utils as utils
from matador import git, zippey


@click.version_option(message='%(prog)s version %(version)s')
@click.group()
def matador():
    pass


@matador.command(name='init')
@click.option('--project', '-p', prompt='Project Name')
def create_project(project):
    cookiecutter(
        'https://github.com/Empiria/matador-cookiecutter.git',
        no_input=True,
        extra_context={'project_name': project})
    click.echo(f'Created matador project {project}')


@matador.command(name='create-ticket')
@click.option('--ticket', '-t', prompt='Ticket')
def create_ticket(ticket):
    project_repo = Repo.discover()
    ticket_folder = Path(project_repo.path, 'deploy', 'tickets', ticket)
    Path.mkdir(ticket_folder, parents=True, exist_ok=True)

    for file_name in ('deploy.py', 'remove.py'):
        file = Path(ticket_folder, file_name)
        with file.open('w') as f:
            f.write('from matador.deployment import *\n\n')
        git.stage_file(project_repo, file)

    git.commit(project_repo, f'Create ticket {ticket}')
    click.echo(f'Created ticket {ticket}')


@matador.command(name='create-package')
@click.option('--package', '-p', prompt='Package')
def create_package(package):
    project_repo = Repo.discover()
    package_folder = Path(project_repo.path, 'deploy', 'packages', package)
    Path.mkdir(package_folder, parents=True, exist_ok=True)

    package_file = Path(package_folder, 'tickets.yml')
    with package_file.open('w') as f:
        f.write(
            '# List each ticket on a separate line preceded by - . e.g.\n')
        f.write('# - 30\n')
        f.write('# - 31\n')
    git.stage_file(project_repo, package_file)

    remove_file = Path(package_folder, 'remove.py')
    with remove_file.open('w') as f:
        f.write('from matador.deployment import *\n\n')

    git.stage_file(project_repo, remove_file)
    git.commit(project_repo, f'Create package {package}')
    click.echo(f'Created package {package}')


@matador.command(name='add-t2p')
@click.option('--ticket', '-t', prompt='Ticket')
@click.option('--package', '-p', prompt='Package')
def add_ticket_to_package(ticket, package):
    project_repo = Repo.discover()
    package_file = Path(
        project_repo.path, 'deploy', 'packages', package, 'tickets.yml')

    with package_file.open('a') as f:
        f.write(f'- {ticket}\n')

    git.stage_file(project_repo, package_file)
    git.commit(project_repo, f'Add ticket {ticket} to package {package}')
    click.echo(f'Added ticket {ticket} to package {package}')


@matador.command(name='deploy-ticket')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--ticket', '-t', prompt='Ticket')
@click.option('--commit', '-c', default=None)
@click.option('--packaged', '-p', is_flag=True, default=False)
@deploys_changes
def deploy_ticket(environment, ticket, commit, packaged):
    click.echo(f'Deploying ticket {ticket} to {environment}')
    deployment_folder = utils.ticket_deployment_folder(
        environment, ticket, commit, packaged)
    source_file = Path(deployment_folder, 'deploy.py')
    SourceFileLoader('deploy', str(source_file)).load_module()


@matador.command(name='remove-ticket')
@click.option('--ticket', '-t', prompt='Ticket')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--commit', '-c', default=None)
@click.option('--packaged', '-p', is_flag=True, default=False)
@deploys_changes
def remove_ticket(environment, ticket, commit, packaged):
    click.echo(f'Removing ticket {ticket} from {environment}')
    deployment_folder = utils.ticket_deployment_folder(
        environment, ticket, commit, packaged)
    source_file = Path(deployment_folder, 'remove.py')
    SourceFileLoader('remove', str(source_file)).load_module()


@matador.command(name='deploy-package')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--package', '-p', prompt='Package')
@click.option('--commit', '-c', prompt='Commit Ref')
def deploy_package(environment, package, commit):
    click.echo(f'Deploying package {package} to {environment}')
    tickets_file = utils.package_definition(environment, package, commit)
    with tickets_file.open('r') as f:
        for ticket in yaml.load(f):
            click.echo('*' * 25)
            click.echo(f'Deploying ticket {ticket} to {environment}')
            deployment_folder = utils.ticket_deployment_folder(
                environment, ticket, commit, True)
            source_file = Path(deployment_folder, 'deploy.py')
            SourceFileLoader('deploy', str(source_file)).load_module()
            click.echo('')


@matador.command(name='remove-package')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--package', '-p', prompt='Package')
@click.option('--commit', '-c', prompt='Commit Ref')
def remove_package(environment, package, commit):
    click.echo(f'Removing package {package} from {environment}')
    tickets_file = utils.package_definition(environment, package, commit)
    with tickets_file.open('r') as f:
        for ticket in yaml.load(f):
            click.echo('*' * 25)
            click.echo(f'Removing ticket {ticket} from {environment}')
            deployment_folder = utils.ticket_deployment_folder(
                environment, ticket, commit, True)
            source_file = Path(deployment_folder, 'deploy.py')
            SourceFileLoader('remove', str(source_file)).load_module()
            click.echo('')


@matador.command(name='run-sql-script')
@click.option('--environment', '-e', prompt='Environment')
@click.argument('file', type=click.File('r'))
@deploys_changes
def run_sql_script(environment, file):
    click.echo(f'Executing {file} against {environment}')
    kwargs = {
        **utils.environment()[environment]['database'],
        **utils.credentials()[environment]
    }
    kwargs['directory'] = str(Path(file).parent),
    kwargs['file'] = str(Path(file).name)
    run_sql_script(**kwargs)


@matador.command(name='smudge-zip')
@click.argument('input', type=click.File('rb'))
@click.argument('output', type=click.File('wb'))
def smudge_zip(input, output):
    input = io.open(sys.stdin.fileno(), 'rb')
    output = io.open(sys.stdout.fileno(), 'wb')
    zippey.init()
    zippey.decode(input, output)


@matador.command(name='clean-zip')
@click.argument('input', type=click.File('rb'))
@click.argument('output', type=click.File('wb'))
def clean_zip(input, output):
    input = io.open(sys.stdin.fileno(), 'rb')
    output = io.open(sys.stdout.fileno(), 'wb')
    zippey.init()
    zippey.encode(input, output)


@matador.command(name='start-service')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--service', '-s', prompt='Service')
@windows_only
def start_service(environment, service):
    click.echo(f'Starting {service} on {environment}')
    from matador.cli import abw_service
    services = utils.environment()[environment]['services']
    abw_service.start(service, services[service])


@matador.command(name='restart-service')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--service', '-s', prompt='Service')
@windows_only
def restart_service(environment, service):
    click.echo(f'Restarting {service} on {environment}')
    from matador.cli import abw_service
    services = utils.environment()[environment]['services']
    abw_service.restart(service, services[service])


@matador.command(name='stop-service')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--service', '-s', prompt='Service')
@windows_only
def stop_service(environment, service):
    click.echo(f'Stopping {service} on {environment}')
    from matador.cli import abw_service
    services = utils.environment()[environment]['services']
    abw_service.stop(service, services[service])


@matador.command(name='service-status')
@click.option('--environment', '-e', prompt='Environment')
@click.option('--service', '-s', prompt='Service')
@windows_only
def service_status(environment, service):
    from matador.cli import abw_service
    services = utils.environment()[environment]['services']
    is_running = abw_service.is_running(services[service])
    click.echo(abw_service.is_running_message(is_running, service))



#!/usr/bin/env python

#  Copyright (c) 2014, Sippey Fun Lab
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#    * Neither the name of the Sippey Fun Lab nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


#
#  Zippey: A Git filter for friendly handling of ZIP-based files
#
#  There are many types of ZIP-based files, such as  Microsoft Office .docx,
#  .xlsx, .pptx files, OpenOffice .odt files and jar files, that contains
#  plaintext content but not really tractable by git due to compression smears
#  parts that have been modified and parts that remain the same across commit.
#  This prevent Git from versioning these files and treat them as a new binary
#  blob every time the file is saved.
#
#  Zippey is a Git filter that un-zip zip-based file into a simple text format
#  during git add/commit ("clean" process) and recover the original zip-based
#  file after git checkout ("smudge" process). Since diff is taken on the
#  "cleaned" file after file is added, it is likely real changes to file can be
#  reflected by original git diff command.
#
#  The text format is defined as a series of records. Each records represent a
#  file in the original zip-based file, which is composed of two parts,
#  a header that contains meta file and a body that contains data. The header
#  is a few data fields segmented by pipe character like this:
#
#       length|raw_length|type|filename
#
#  where length is an ascii coded integer of the following data section, raw_length
#  is the orginal length of data (if transformation is taken), type can be A for
#  text data or B for binary data, and filename is the original file name
#  including path if the zip-based file contains directories. Immediately after
#  the header, there is a carriage return ('\n'), follows "length" byte of
#  data, and then another CR and then the next recor, i,e,
#
#       [header1]\n[data1]\n[header2]\n[data2] ...
#
#  There are two types of data section. If the file contains only text data,
#  its content is copied to data section without any change, otherwise, data
#  is base64 coded to ensure the entire file is text format.
#
#
#  Author: Sippey (sippey@gmail.com)
#  Date: Apr.18, 2014
#
#  Modified by Kristian Hoey Horsberg <khh1990 ' at ' gmail.com>
#  to make python 3 compatible
#  Date May 20th 2014
#
# PEP8 and spelling errors corrected
# Modified for inclusion within matador
# by Owen Campbell 23 Feb 2016

import zipfile
import sys
import base64
import string
import tempfile
import os.path

DEBUG_ZIPPEY = False
NAME = 'Zippey'
ENCODING = 'UTF-8'


def debug(msg):
    '''Print debug message'''
    if DEBUG_ZIPPEY:
        sys.stderr.write('{0}: debug: {1}\n'.format(NAME, msg))


def error(msg):
    '''Print error message'''
    sys.stderr.write('{0}: error: {1}\n'.format(NAME, msg))


def init():
    '''Initialize writing; set binary mode for windows'''
    debug("Running on {}".format(sys.platform))
    if sys.platform.startswith('win'):
        import msvcrt
        debug("Enable Windows binary workaround")
        msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
        msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)


def encode(input, output):
    '''Encode into special VCS friendly format from input to output'''
    debug("ENCODE was called")
    tfp = tempfile.TemporaryFile(mode='w+b')
    tfp.write(input.read())
    zfp = zipfile.ZipFile(tfp, "r")
    for name in zfp.namelist():
        data = zfp.read(name)
        text_extensions = ['.txt', '.html', '.xml']
        extension = os.path.splitext(name)[1][1:].strip().lower()
        try:
            # Check if text data
            data.decode(ENCODING)
            try:
                strdata = map(chr, data)
            except TypeError:
                strdata = data
            if extension not in text_extensions and not all(
                c in string.printable for c in strdata
            ):
                raise UnicodeDecodeError(
                    ENCODING, "".encode(ENCODING), 0, 1,
                    "Artificial exception")

            # Encode
            debug("Appending text file '{}'".format(name))
            output.write("{}|{}|A|{}\n".format(
                len(data), len(data), name).encode(ENCODING))
            output.write(data)
            output.write("\n".encode(ENCODING))
        except UnicodeDecodeError:
            # Binary data
            debug("Appending binary file '{}'".format(name))
            raw_len = len(data)
            data = base64.b64encode(data)
            output.write("{}|{}|B|{}\n".format(
                len(data), raw_len, name).encode(ENCODING))
            output.write(data)
            output.write("\n".encode(ENCODING))
    zfp.close()
    tfp.close()


def decode(input, output):
    '''Decode from special VCS friendly format from input to output'''
    debug("DECODE was called")
    tfp = tempfile.TemporaryFile(mode='w+b')
    zfp = zipfile.ZipFile(tfp, "w", zipfile.ZIP_DEFLATED)

    while True:
        meta = input.readline().decode(ENCODING)
        if not meta:
            break

        (data_len, raw_len, mode, name) = [
            t(s) for (t, s) in zip((int, int, str, str), meta.split('|'))]
        if mode == 'A':
            debug("Appending text file '{}'".format(name))
            zfp.writestr(name.rstrip(), input.read(data_len))
            input.read(1)  # Skip last '\n'
        elif mode == 'B':
            debug("Appending binary file '{}'".format(name.rstrip()))
            zfp.writestr(name.rstrip(), base64.b64decode(input.read(data_len)))
            input.read(1)  # Skip last '\n'
        else:
            # Should never reach here
            zfp.close()
            tfp.close()
            error('Illegal mode "{}"'.format(mode))
            sys.exit(1)

    # Flush all writes
    zfp.close()

    # Write output
    tfp.seek(0)
    output.write(tfp.read())
    tfp.close()



import click
import shutil
import yaml
from dulwich.repo import Repo
from pathlib import Path
from dulwich.errors import NotGitRepository
from configparser import ConfigParser
from matador import git


def deployment_repository(project_folder):
    project = Path(project_folder).name
    deployment_folder = Path(Path.home(), '.matador', project)
    deployment_repo_folder = Path(deployment_folder, 'repository')

    Path.mkdir(deployment_folder, parents=True, exist_ok=True)
    Path.mkdir(deployment_repo_folder, parents=True, exist_ok=True)

    try:
        repo = Repo(str(deployment_repo_folder))
    except NotGitRepository:
        config_file = Path(deployment_repo_folder, '.git', 'config')
        config = ConfigParser()

        repo = Repo.init(str(deployment_repo_folder))
        config.read(str(config_file))

        config['core']['sparsecheckout'] = 'true'
        config['remote "origin"'] = {
            'url': project_folder.as_posix(),
            'fetch': '+refs/heads/*:refs/remotes/origin/*'
        }

        with config_file.open('w') as f:
            config.write(f)

        sparse_checkout_file = Path(
            deployment_repo_folder, '.git', 'info', 'sparse-checkout')
        with sparse_checkout_file.open('a') as f:
            f.write('/src\n')
            f.write('/deploy\n')

    return repo


def ticket_deployment_folder(environment, ticket, commit, packaged):
    project_repo = Repo.discover()
    project_folder = Path(project_repo.path)
    project = project_folder.name
    ticket_deployment_folder = Path(
        Path.home(), '.matador', project, environment, 'tickets', ticket)
    if commit is None:
        commit = project_repo.head().decode(encoding='ascii')
    deployment_repo = deployment_repository(project_folder)
    ticket_src_folder = Path(deployment_repo.path, 'deploy', 'tickets', ticket)
    if not packaged:
        git.fetch_all(project_repo, deployment_repo)

    shutil.rmtree(str(ticket_deployment_folder), ignore_errors=True)
    git.checkout(deployment_repo, commit)
    shutil.copytree(str(ticket_src_folder), str(ticket_deployment_folder))
    return ticket_deployment_folder


def package_definition(environment, package, commit):
    project_repo = Repo.discover()
    project_folder = Path(project_repo.path)
    project = project_folder.name
    package_deployment_folder = Path(
        Path.home(), '.matador', project, environment, 'packages', package)
    deployment_repo = deployment_repository(project_folder)
    package_src_folder = Path(
        deployment_repo.path, 'deploy', 'packages', package)

    git.fetch_all(project_repo, deployment_repo)
    shutil.rmtree(str(package_deployment_folder), ignore_errors=True)
    git.checkout(deployment_repo, commit)
    shutil.copytree(str(package_src_folder), str(package_deployment_folder))
    tickets_file = Path(package_deployment_folder, 'tickets.yml')
    return tickets_file


def environments():
    project_folder = Path(Repo.discover().path)
    file_path = Path(
        project_folder, 'config', 'environments.yml')
    try:
        file = file_path.open('r')
        environments = yaml.load(file)
        if environments:
            return environments
        else:
            raise ValueError()
    except FileNotFoundError:
        click.echo('Cannot find environments.yml file')
    except ValueError:
        click.echo('environments.yml exists but is empty')


def credentials():
    project_folder = Path(Repo.discover().path)
    file_path = Path(
        project_folder, 'config', 'credentials.yml')
    try:
        file = file_path.open('r')
        credentials = yaml.load(file)
        if credentials:
            return credentials
        else:
            raise ValueError()
    except FileNotFoundError:
        click.echo('Cannot find credentials.yml file')
    except ValueError:
        click.echo('credentials.yml exists but is empty')



from pathlib import Path
from collections import defaultdict
import os
import subprocess
from string import Template


def command_condition(*args):
    """

    Parameters
    ----------
    args: list
        containing boolean values for:
            if a command has been specified explicitly
            if the dbms is oracle
            if the os is posix
            if windows authentication has been specified

    Returns
    -------
        tuple of values which can be used to determine the appropriate command
        to run an sql script

    """
    try:
        most_significant_argument = args.index(True)
    except ValueError:
        most_significant_argument = -1

    conditions = {
        0: ('command'),
        1: ('oracle'),
        2: ('mssql', 'posix'),
        3: ('mssql', 'nt', 'windows_authentication'),
        -1: ('mssql', 'nt', 'mssql_authentication')
    }

    return conditions[most_significant_argument]


def command(**kwargs):
    """
    Parameters
    ----------
    kwargs : dict
        A dictionary containing values for:
            command (optional, overrides all other entries)
            dbms
            client_os
            server
            db_name
            port (Oracle only)
            windows_authentication (MSSQL on windows clients only)
            user
            password
    """
    condition = command_condition(
        'command' in kwargs,
        kwargs['dbms'].lower() == 'oracle',
        kwargs['client_os'] == 'posix',
        'windows_authentication' in kwargs)

    # Create a default dictionary based on kwargs but with any missing
    # keys returning an empty string.
    params = defaultdict(str, kwargs)

    commands = {
        ('command'):
            params['command'],
        ('oracle'):
            'sqlplus -S -L ${user}/${password}@${server}:${port}/${db_name}',
        ('mssql', 'posix'):
            'bsqldb -S ${server} -D ${db_name} -U ${user} -P ${password}',
        ('mssql', 'nt', 'windows_authentication'):
            'sqlcmd -S ${server} -d ${db_name} -E',
        ('mssql', 'nt', 'mssql_authentication'):
            'sqlcmd -S ${server} -d ${db_name} -U ${user} -P ${password}'
    }

    return Template(commands[condition]).substitute(params)


def sql_script(**kwargs):
    """
    Parameters
    ----------
    kwargs : dict
        A dictionary containing values for:
            dbms
            directory
            file
    """
    file = Path(kwargs['directory'], kwargs['file'])

    with file.open('r', encoding='utf8') as f:
        script = f.read()
        f.close()

    if kwargs['dbms'] == 'oracle':
        script += '\nshow error'

    return script.encode('utf-8')


def run_sql_script(**kwargs):
    """
    Parameters
    ----------
    kwargs : dict
        A dictionary containing values for:
            dbms
            server
            db_name
            directory
            file
    """
    kwargs['client_os'] = os.name
    cwd = os.getcwd()

    os.chdir(kwargs['directory'])

    process = subprocess.Popen(
        command(**kwargs).split(),
        stdin=subprocess.PIPE)
    process.stdin.write(sql_script(**kwargs))
    process.stdin.close()
    process.wait()

    os.chdir(cwd)



import functools
import os
import click
from pathlib import Path
from dulwich.repo import Repo
import matador.session as session


def windows_only(func):
    @functools.wraps(func)
    def wrapped(*args, **kwargs):
        if os.name != 'nt':
            click.echo(f'{func.__name__} can only be used on a Windows system')
        else:
            return func(*args, **kwargs)
    return wrapped


def deploys_changes(func):
    """Creates sub-directories in the .matador directory for the specific
    environment used by the wrapped function"""
    @functools.wraps(func)
    def wrapped(*args, **kwargs):
        project = Path(Repo.discover().path).name
        environment_folder = Path(
            Path.home(), '.matador', project, kwargs['environment'])
        tickets_folder = Path(environment_folder, 'tickets')
        packages_folder = Path(environment_folder, 'packages')

        if 'ticket' in kwargs:
            session.environment = kwargs['environment']
            session.ticket = kwargs['ticket']

        Path.mkdir(environment_folder, parents=True, exist_ok=True)
        Path.mkdir(tickets_folder, parents=True, exist_ok=True)
        Path.mkdir(packages_folder, parents=True, exist_ok=True)
        return func(*args, **kwargs)
    return wrapped




import win32serviceutil as win32
import matador.cli.utils as utils
import matador.session as session


def is_running(name):
    environment = utils.environments[session.environment]
    server = environment['abwServer']
    return win32.QueryServiceStatus(name, machine=server)[1] == 4


def is_running_message(is_running, label):
    if is_running:
        return f'{label} - Running on {session.environment}'
    else:
        return f'{label} - Not running on {session.environment}'


def start(label, name):
    environment = utils.environments[session.environment]
    server = environment['abwServer']
    if not is_running(name):
        win32.StartService(name, machine=server)
    message = is_running_message(is_running(name), label)
    return message


def stop(label, name):
    environment = utils.environments[session.environment]
    server = environment['abwServer']
    if is_running(name):
        win32.StopService(name, machine=server)
    message = is_running_message(is_running(name), label)
    return message


def restart(label, name):
    environment = utils.environments[session.environment]
    server = environment['abwServer']
    if is_running(name):
        win32.RestartService(name, machine=server)
    message = is_running_message(is_running(name), label)
    return message



import click
import os
from dulwich.repo import Repo
from pathlib import Path
from matador import git
from matador import zippey
import matador.session as session
import matador.cli.utils as utils
import shutil
from openpyxl import load_workbook


def _create_deployment_xlsx_file(source_file, deployment_file, commit_ref):
    """Copy a source .xlsx file to the deployment folder and add git info its
    file properties.
    """
    project = Path(Repo.discover().path).name
    repo_folder = Path(Path.home(), '.matador', project, 'repository')
    repo = Repo(str(repo_folder))

    version, commit_timestamp, author = git.keyword_values(repo, commit_ref)

    deployment_file.touch()
    zippey.decode(source_file.open('rb'), deployment_file.open('wb'))

    workbook = load_workbook(str(deployment_file))
    workbook.properties.creator = author
    workbook.properties.version = version
    workbook.save(str(deployment_file))


def _create_deployment_text_file(source_file, deployment_file, commit_ref):
    """Copy a source text file to the deployment folder and perform git keyword
    substitution on the copy.
    """
    project = Path(Repo.discover().path).name
    repo_folder = Path(Path.home(), '.matador', project, 'repository')
    repo = Repo(str(repo_folder))

    with source_file.open('r') as f:
        original_text = f.read()

    new_text = git.substitute_keywords(original_text, repo, commit_ref)

    with deployment_file.open('w') as f:
        f.write(new_text)


create_deployment_file = {
    '.xlsx': _create_deployment_xlsx_file,
    '.arw': _create_deployment_text_file,
    '.rpx': _create_deployment_text_file,
}


def deploy_report_file(report_name, report_file_name, commit_ref):
    """Checkout a report file from the matador repo, copy it to the deployment
    folder, add git keywords to the copy and deploy the result to the ABW
    Customised Reports folder.
    """
    project = Path(Repo.discover().path).name
    repo_folder = Path(Path.home(), '.matador', project, 'repository')
    repo = Repo(str(repo_folder))
    deployment_folder = Path(
        Path.home(), '.matador', project, session.environment, 'tickets',
        session.ticket)

    source_file = Path(
        repo_folder, 'src', 'reports', report_name, report_file_name)
    deployment_file = Path(deployment_folder, report_file_name)

    # I've been unable to get UNC shares working as Path objects, so I'm
    # using a simple string here. Also, that's what shutil requires anyway.
    environment = utils.environments()[session.environment]
    target_folder = (
        '//' +
        environment['abwServer'] + '/' +
        environment['customisedReports'])

    git.checkout(repo, commit_ref)
    create_deployment_file[source_file.suffix](
        source_file, deployment_file, commit_ref)
    click.echo(f'Deploying {report_file_name} to {target_folder}')
    shutil.copy(str(deployment_file), target_folder)


def remove_report_file(report_file_name):
    """Remove a report file from the ABW Customised Reports Folder"""
    environment = utils.environments()[session.environment]
    target_folder = (
        '//' +
        environment['abwServer'] + '/' +
        environment['customisedReports'])
    target_file = target_folder + '/' + report_file_name
    click.echo(f'Removing {report_file_name} from {target_folder}')
    try:
        os.remove(target_file)
    except FileNotFoundError:
        click.echo(f'{report_file_name} does not exist in {target_folder}')



import click
from pathlib import Path
from dulwich.repo import Repo
from matador.cli.sql import run_sql_script
from matador import git
import matador.session as session
import matador.cli.utils as utils


def _fetch_script(repo, script_path, commit_ref, target_folder):
    script = Path(repo.path, script_path)
    target_script = Path(target_folder, script.name)

    git.checkout(repo, commit_ref)

    with script.open('r') as f:
        original_text = f.read()
        f.close()

    new_text = git.substitute_keywords(original_text, repo, commit_ref)

    with target_script.open('w') as f:
        f.write(new_text)
        f.close()

    return target_script


def deploy_sql_script(path, commit_ref=None):
    """Checkout an sql script from the matador repo and execute it against
    the database server defined for the environment.
    """
    script_path = Path(path)
    click.echo(f'Executing {script_path.name} against {session.environment}')
    project = Path(Repo.discover().path).name
    deployment_folder = Path(
        Path.home(), '.matador', project, session.environment, 'tickets',
        session.ticket)
    if str(script_path.parent) == '.':
        script = Path(deployment_folder, script_path)
    else:
        repo_folder = Path(Path.home(), '.matador', project, 'repository')
        repo = Repo(str(repo_folder))
        script = _fetch_script(repo, script_path, commit_ref, deployment_folder)

    kwargs = {
        **utils.environments()[session.environment]['database'],
        **utils.credentials()[session.environment]
    }
    kwargs['directory'] = str(script.parent)
    kwargs['file'] = str(script.name)

    run_sql_script(**kwargs)


def deploy_oracle_package(package_name, commit_ref=None):
    """Checkout the scripts for a package body and spec and execute them
    against the database server defined for the environment.
    """
    project = Path(Repo.discover().path).name
    deployment_folder = Path(
        Path.home(), '.matador', project, session.environment, 'tickets',
        session.ticket)
    repo_folder = Path(Path.home(), '.matador', project, 'repository')
    repo = Repo(str(repo_folder))
    package_folder = Path(
        repo_folder, 'src', 'db_objects', 'packages', package_name)
    package_spec = Path(package_folder, package_name + '.pks')
    package_body = Path(package_folder, package_name + '.pkb')

    spec_script = _fetch_script(repo, package_spec, commit_ref, deployment_folder)
    body_script = _fetch_script(repo, package_body, commit_ref, deployment_folder)

    kwargs = {
        **utils.environments()[session.environment]['database'],
        **utils.credentials()[session.environment]
    }
    kwargs['directory'] = str(spec_script.parent)
    kwargs['file'] = str(spec_script.name)

    run_sql_script(**kwargs)

    kwargs['file'] = str(body_script.name)
    run_sql_script(**kwargs)
